#include "common.h"
#include "tiny_printf.h"
#include "fs.h"
#include "gps.h"
#include "main.h"

const char *kml_header = "\
<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\
<kml xmlns=\"http://earth.google.com/kml/2.2\">\n\
<Document>\n\
 <name>GPS logger</name>\n\
 <Style id=\"line\">\n\
  <LineStyle>\n\
   <color>FF3300BB</color>\n\
   <width>3</width>\n\
  </LineStyle>\n\
 </Style>\n\
 <Style id=\"marker\">\n\
  <IconStyle>\n\
   <Icon>\n\
    <href>http://www.google.com/mapfiles/marker.png</href>\n\
   </Icon>\n\
  </IconStyle>\n\
 </Style>\n";

const char *kml_footer = "\
</Document>\n\
</kml>\n";

const char *trace_header1 = "\
 <Placemark>\n\
  <name>";

const char *trace_header2 = "\
</name>\n\
  <description>";

const char *trace_header3 = "\
</description>\n\
  <styleUrl>#line</styleUrl>\n\
  <LineString>\n\
   <tessellate>1</tessellate>\n\
   <coordinates>\n";

const char *trace_footer = "\
   </coordinates>\n\
  </LineString>\n\
 </Placemark>\n";

const char *mark_header1 = "\
 <Placemark>\n\
  <name>";

const char *mark_header2 = "\
</name>\n\
  <description>";

const char *mark_header3 = "\
</description>\n\
  <styleUrl>#marker</styleUrl>\n\
  <Point>\n\
   <coordinates>";

const char *mark_footer = "\
</coordinates>\n\
  </Point>\n\
 </Placemark>\n";

unsigned char kmllog_working =0;
char kmllog_name[13] = "";
unsigned short kmllog_mark_num;
static char buff[32];
struct gps_coord_st prev_latitude, prev_longitude;

void kmllog_start(void)
{
	unsigned char rv;

	kmllog_mark_num = 1;
	memset((char *)&prev_latitude, 0, sizeof(prev_latitude));
	memset((char *)&prev_longitude, 0, sizeof(prev_longitude));

	// Create filename MMDDhhmm.kml
	tiny_printf(NULL, kmllog_name, "%02d%02d%02d%02d.kml",
		gps_info.daytime.mon,
		gps_info.daytime.day,
		gps_info.daytime.hour,
		gps_info.daytime.min);
	
	// Init FAT16 filesystem
	rv = fs_init();
	switch(rv){
	case FS_OK:
		break;
	case FS_NOCARD:
		disp_status("Error", "No SD card");
		return;
	case FS_NOTFAT16:
		disp_status("Error", "Not FAT16 format");
		return;
	default:
		disp_status("Error", "fs_init()");
		return;
	}

	// Open new file
	rv = fs_open(kmllog_name, FS_MODE_WR);
	switch(rv){
	case FS_OK:
		break;
	case FS_FULL:
		disp_status("Error", "SD Card full");
		return;
	default:
		disp_status("Error", "fs_open()");
		return;
	}
	
	// Write kml header
	rv = fs_write(kml_header, strlen(kml_header));
	if(rv) goto error;

	// Write trace header
	rv = fs_write(trace_header1, strlen(trace_header1));
	if(rv) goto error;

	// Write trace name here
	tiny_printf(NULL, buff, "Trace %hd", kmllog_mark_num); 
	rv = fs_write(buff, strlen(buff));
	if(rv) goto error;

	rv = fs_write(trace_header2, strlen(trace_header2));
	if(rv) goto error;

	// Write trace description here
	tiny_printf(NULL, buff, "Started at %02d/%02d/%02d %02d:%02d:%02d",
		gps_info.daytime.year,
		gps_info.daytime.mon,
		gps_info.daytime.day,
		gps_info.daytime.hour,
		gps_info.daytime.min,
		gps_info.daytime.sec );
	rv=fs_write(buff, strlen(buff));
	if(rv) goto error;

	rv = fs_write(trace_header3, strlen(trace_header3));
	if(rv) goto error;
	// Write header done

	disp_status("Info", "Log started");
	kmllog_working = 1;
	return;

error:
	disp_status("Error", "fs_write()");
}

void kmllog_stop(void)
{
	if(kmllog_working == 0) return;
	fs_write(trace_footer, strlen(trace_footer));
	fs_write(kml_footer, strlen(kml_footer));
	fs_close();
	disp_status("Info", "Log stopped");
	kmllog_working = 0;
}



void kmllog_point(unsigned char force_plot)
{
	double dlong_sub, dlat_sub;
	long long_sub, lat_sub;
	unsigned char rv;

	if(kmllog_working ==0) return;

#if 0
	if( !memcmp((const char *)&gps_info.latitude, (const char *)&prev_latitude, sizeof(struct gps_coord_st)) 
	  && !memcmp((const char *)&gps_info.longitude, (const char *)&prev_longitude, sizeof(struct gps_coord_st))
	  && !force_plot ){
		prev_longitude = gps_info.longitude;
		prev_latitude = gps_info.latitude;
		return;
	}
#endif

	// Convert 59.9999 scale to 99.9999 scale
	dlong_sub = ((double)gps_info.longitude.min) * 10000 + gps_info.longitude.submin;
	dlat_sub = ((double)gps_info.latitude.min) * 10000 + gps_info.latitude.submin;
	dlong_sub = dlong_sub * (double)1.666667;
	dlat_sub = dlat_sub * (double)1.666667;
	long_sub = (long)dlong_sub;
	if(long_sub > 999999) long_sub = 999999;
	lat_sub = (long)dlat_sub;
	if(lat_sub > 999999) lat_sub = 999999;

	// Write coordinate
	tiny_printf(NULL, buff, "%02hd.%06ld,%02hd.%06ld,%hd.%hd\n",
		gps_info.longitude.deg, long_sub,					// longitude
		gps_info.latitude.deg, lat_sub,						// latitude
		gps_info.altitude.meter, gps_info.altitude.submeter);	// altitude

	rv = fs_write(buff, strlen(buff));
	if(rv){
		disp_status("Error", "fs_write()");
		kmllog_working = 0;
	}

	prev_longitude = gps_info.longitude;
	prev_latitude = gps_info.latitude;
}


void kmllog_mark(void)
{
	double dlong_sub, dlat_sub;
	long long_sub, lat_sub;
	unsigned char rv;

	if(kmllog_working ==0) return;

	tiny_printf(NULL, buff, "Mark %hd", kmllog_mark_num);
	disp_status("Info", buff);

	// End current trace
	rv = fs_write(trace_footer, strlen(trace_footer));
	if(rv) goto error;

	// Write marker
	rv = fs_write(mark_header1, strlen(mark_header1));
	if(rv) goto error;

	// Write mark name here
	tiny_printf(NULL, buff, "Mark %hd", kmllog_mark_num);
	rv = fs_write(buff, strlen(buff));
	if(rv) goto error;

	rv = fs_write(mark_header2, strlen(mark_header2));
	if(rv) goto error;

	// Write mark description here
	tiny_printf(NULL, buff, "Marked at %02d/%02d/%02d %02d:%02d:%02d\r",
		gps_info.daytime.year,
		gps_info.daytime.mon,
		gps_info.daytime.day,
		gps_info.daytime.hour,
		gps_info.daytime.min,
		gps_info.daytime.sec );
	rv=fs_write(buff, strlen(buff));
	if(rv) goto error;

	// Write current altitude and speed
	tiny_printf(NULL, buff, "Alt %hd.%hd m  Spd %hd.%hd km/h",
		gps_info.altitude.meter,
		gps_info.altitude.submeter,
		gps_info.speed.meter,
		gps_info.speed.submeter/10 );
	rv=fs_write(buff, strlen(buff));
	if(rv) goto error;
	
	rv = fs_write(mark_header3, strlen(mark_header3));
	if(rv) goto error;
	
	// Write coordinate
	kmllog_point(1);

	rv = fs_write(mark_footer, strlen(mark_footer));
	if(rv) goto error;

	// Start new trace
	kmllog_mark_num ++;
	rv = fs_write(trace_header1, strlen(trace_header1));
	if(rv) goto error;

	// Write trace name here
	tiny_printf(NULL, buff, "Trace %hd", kmllog_mark_num); 
	rv = fs_write(buff, strlen(buff));
	if(rv) goto error;

	rv = fs_write(trace_header2, strlen(trace_header2));
	if(rv) goto error;

	// Write trace description here
	tiny_printf(NULL, buff, "Started at %02d/%02d/%02d %02d:%02d:%02d",
		gps_info.daytime.year,
		gps_info.daytime.mon,
		gps_info.daytime.day,
		gps_info.daytime.hour,
		gps_info.daytime.min,
		gps_info.daytime.sec );
	rv=fs_write(buff, strlen(buff));
	if(rv) goto error;

	rv = fs_write(trace_header3, strlen(trace_header3));
	if(rv) goto error;

	fs_sync();

	return;

error:
	disp_status("Error", "fs_write()");
	kmllog_working =0;
}
